@@ -78,7 +78,7 @@ class TaskType(str, Enum):
     retrieval = "retrieval"
     cloze = "cloze"
     cloze_multiple_choice = "cloze-multiple-choice"
-    cloze_hint = "cloze-hint"
+    cloze_generative = "cloze-generative"
     text_editing = "text-editing"
     grammatical_error_correction = "grammatical-error-correction"
     essay_writing = "essay-writing"
